function [ mImageApprox, nChosen, vCn, iV1, iV2, iMDictionary2D] = ApproxDCT(mImage, approxCriteria, thresholding)
% ApproxDCT Approximates a \(2\)D signal by thresholding the smallest DCT coefficients.

if nargin < 3
    thresholding = 1;
end
szMImage = size(mImage);
nPixels = szMImage(1)*szMImage(2);
%% Get the dct coeffieients
mCn = dct2(mImage);

%% Perform approximation
if thresholding
    
    [ mCnApprox, nChosen, vCn, iV1, iV2 ] = Thresholding( mCn, approxCriteria );
    
else
    
    [ mCnApprox, nChosen, vCn, iV1, iV2 ] = ParsevalApprox( mCn, approxCriteria );
    
end

%% Use coefficients to calculate approximation
% If all coefficients or no coefficients are picked don't perform reconstruction
if nChosen == nPixels
    mImageApprox = mImage;
elseif nChosen == 0
    mImageApprox = zeros(szMImage);
    iV1 = 1;
    iV2 = 1;
    nChosen = 1;
    vCn = 0;
else
    % Reconstruct the image        
    mImageApprox = idct2(mCnApprox);
end

% Get the index in the 2D seperable dictionary
iMDictionary2D = KroneckerIndex(1,1,iV2',iV1',size(mImage,1));