% OMP2D Orthogonal Matching Pursuit in 2D
%
% It creates an atomic decomposition of a \(2\)D signal using OMP criterion and assuming
% separable dictionaries. You can choose a tolerance, the number of atoms to take in or an 
% initial subspace to influence the OMP algorithm. 
%  
% Usage:    [H,Di1,Di2, beta, c,Q ] = OMP2D( f, Dx,Dy, tol, No, ind);
%           H = OMP2D(f,Dx,Dy); variables tol, No, can also be []
%                 
% Inputs:
%   f       Analyzing \(2\)D signal (Image).
%   Dx      Dictionary of normalized atoms (w.r.t. Image rows).
%   Dy      Dictionary of normalized atoms (w.r.t. Image columns).
%   tol     Desired distance between \(f\) and its approximation \(H\).
%           Default=\(6.5\).
%   No      (optional) maximal number of atoms to choose, if the number of chosen atoms
%           equals to No, routine will stop (default numel(\(f\))).
%   indx    (optional) indices for an initial subspace; They operate as indx(k),indy(k).
%   indy    (optional) indices determining the initial subspace (as above).
%
% Outputs:
%   H       Approximation of \(f\)
%   Di1     Indices of selected atoms w.r.t the original Dx
%   Di2     Indices of selected atoms w.r.t the original Dy
%   beta    Biorthogonal atoms corresponding to the \(2\)D selected atoms 
%   c       Coefficients of the atomic decomposition
%   Q       Orthogonal \(2\)D basis for the selected subspace
%  
% The implementation of OMP method is based on  Gram-Schmidt orthonormalization and adaptive 
% birothogonalization, as proposed in Ref 2 below.
%
%  References:
%
%  1-Y.C. Pati, R. Rezaiifar, and P.S. Krishnaprasad, "Orthogonal matching pursuits: 
%  recursive function approximation with applications to wavelet decomposition", in 
%  Proceedings of the 27th Asilomar Conference on Signals, Systems and Computers, 1993.
%   
%  2-L. Rebollo-Neira and D. Lowe, "Optimized Orthogonal Matching Pursuit Approach", IEEE
%  Signal Processing Letters, Vol(9,4), 137-140, (2002).

% More  information at  http://www.nonlinear-approx.info/