function [ cImageApprox, nCoefficients, cCoefficients, cIndex, nZeroBlocks] = ProcessCellImageGreedy2D(cImageGrey, tolerance, mDictionary, vSize, algorithm)
%  ProcessCellImageGreedy2D Using the supplied 1D dictionary it applies the greedy algorithm
%  specified to every cell in a blocked image.

functionName = mfilename;
%% Input Validation
if nargin < 4
    error('Not enough input arguments!');
elseif nargin < 5
    algorithm = 'OMP';
end

[ nYBlocks, nXBlocks ] = size(cImageGrey);

if nYBlocks == 0 || nXBlocks  == 0
    
    error('%s: The cell reprentation of the image has 0 dimensions!',functionName);   
    
end

if size(cImageGrey{1,1},1) ~= size(cImageGrey{1,1},2)
    
    error('%s: The image blocks are not square!',functionName);
    
end

% If this routine is kept need to validate mDictionary.
%% Initialize return values
cImageApprox = cell(nYBlocks,nXBlocks);
cCoefficients = cell(nYBlocks,nXBlocks);
cIndex = cell(nYBlocks,nXBlocks);
%% Find the sparse approximation of each block
% Set up the dictionary here to avoid calculating it for every block
% mDictionary2D = Dictionary('RDCTDB',squareBlockWidth);
nZeroBlocks = 0;
nCoefficients = 0;
fhandle = str2func(algorithm);
blockWidth = size(cImageGrey{1,1});
for i = 1:nYBlocks
    
    for j = 1:nXBlocks
        
        % Check for zero intensity blocks here
        if sum(cImageGrey{i,j}(:) == 0) == numel(cImageGrey{i,j})
            nZeroBlocks = nZeroBlocks + 1;
            cIndex{i,j}.iMDictionary2D = 1;
            cIndex{i,j}.iD1 = 1;
            cIndex{i,j}.iV1 = 1;
            cIndex{i,j}.iD2 = 1;
            cIndex{i,j}.iV2 = 1;
            cCoefficients{i,j} = 0;   
            nCoefficients = nCoefficients + 1;
            cImageApprox{i,j} = zeros(blockWidth);
            
        else           
            [cImageApprox{i,j},iD1,iD2, beta, cCoefficients{i,j}, Q, iDXY] = fhandle( cImageGrey{i,j}, mDictionary, mDictionary, tolerance);
            nChosen = numel(cCoefficients{i,j});
            % Check that OMP has chosen at least one coefficient
            if nChosen == 0                
                nZeroBlocks = nZeroBlocks + 1;
                cIndex{i,j}.iMDictionary2D = 1;
                cIndex{i,j}.iD1 = 1;
                cIndex{i,j}.iV1 = 1;
                cIndex{i,j}.iD2 = 1;
                cIndex{i,j}.iV2 = 1;
                cCoefficients{i,j} = 0;   
                nCoefficients = nCoefficients + 1;    
            else
                cIndex{i,j}.iMDictionary2D = iDXY;
                cIndex{i,j}.iD1 = 1;
                cIndex{i,j}.iD2 = 1;
                cIndex{i,j}.iV1 = iD1;
                cIndex{i,j}.iV2 = iD2;
                nCoefficients = nCoefficients + nChosen;
            end
        end
    end
    
end    

if nargout < 5 && nZeroBlocks > 0
     
    fprintf('%s: %i zero energy blocks.\n',functionName, nZeroBlocks);
    
end