function [ cImageApprox, nCoefficients, cCoefficients, cIndex, nZeroBlocks] = ProcessCellImageRDCTDB(cImageGrey, tolerance)
%  ProcessCellImageRDCTDB  Applies ApproxRDCTDB to every cell in a blocked image.  

functionName = mfilename;
%% Input Validation
[ nYBlocks, nXBlocks ] = size(cImageGrey);

if nYBlocks == 0 || nXBlocks  == 0
    
    error('%s: The cell reprentation of the image has 0 dimensions!',functionName)
    
else
    
    squareBlockWidth = size(cImageGrey{1,1},1);
    
end

if size(cImageGrey{1,1},1) ~= size(cImageGrey{1,1},2)
    
    error('%s: The image blocks are not square!',functionName)
    
end
%% Initialize return values
cImageApprox = cell(nYBlocks,nXBlocks);
cCoefficients = cell(nYBlocks,nXBlocks);
cIndex = cell(nYBlocks,nXBlocks);
%% Find the sparse approximation of each block
% Set up the dictionary here to avoid calculating it for every block
[mDictionary, vSize, mDictionary2D] = Dictionary('RDCTDB',squareBlockWidth);
nZeroBlocks = 0;
nCoefficients = 0;
for i = 1:nYBlocks
    
    for j = 1:nXBlocks
        
        if sum(cImageGrey{i,j}(:) == 0) == numel(cImageGrey{i,j})
            cImageApprox{i,j} = zeros(size(cImageGrey{i,j}));
            nZeroBlocks = nZeroBlocks + 1;
            cIndex{i,j}.iMDictionary2D = 1;
            cIndex{i,j}.iD1 = 1;
            cIndex{i,j}.iV1 = 1;
            cIndex{i,j}.iD2 = 1;
            cIndex{i,j}.iV2 = 1;
            cCoefficients{i,j} = 0;   
            nCoefficients = nCoefficients + 1;
        else        
            [ cImageApprox{i,j}, cCoefficients{i,j}, iD1, iV1, iD2, iV2, nChosen, iMDictionary2D] =  ...
                ApproxRDCTDB(cImageGrey{i,j}, mDictionary2D, tolerance, squareBlockWidth);    
            if cCoefficients{i,j}(1) == 0
                nZeroBlocks = nZeroBlocks + 1;
            end
            cIndex{i,j}.iD1 = iD1(1:nChosen);
            cIndex{i,j}.iV1 = iV1(1:nChosen);
            cIndex{i,j}.iD2 = iD2(1:nChosen);
            cIndex{i,j}.iV2 = iV2(1:nChosen);
            cIndex{i,j}.iMDictionary2D = iMDictionary2D(1:nChosen);
            nCoefficients = nCoefficients + nChosen;
        end
        
    end
    
end     

if nargout < 5 && nZeroBlocks > 0
     
    fprintf('%s: %i zero energy blocks.\n',functionName, nZeroBlocks);
    
end