function [cImage, mImage, nYBlocks, nXBlocks, y, x] = BlockImage(mImage, blockWidth)
% BlockImage Splits a <latex>2<latex>D signal into nYBlocks x nXBlocks cell of blockWidth x blockWidth
% matrices.
%
% If the matrix cannot be split exactly into blocks of blockWidth x blockWidth then it is
% reduced in size so it can be.


% Usage:        [cImage, mImage, nYBlocks, nXBlocks] = BlockImage(mImage, blockWidth);
%
% Inputs:
%   mImage      Matrix we want to split into blocks
%   blockWidth  Width of the square blocks
%
% Outputs:
%   cImage      Cell containing nYBlocks x nXBlocks of matrices of size 
%               blockWidth x blockWidth.
%   nYBlocks    Number of blocks in the first dimension.
%   nXBlocks    Number of blocks in the second dimension.
%

%% Input Validation
if nargin < 2
    error('The image matrix and blockWidth are mandatory!');
end
%% Check the matrix can be split exactly into square blocks of width blockwidth
[y x] = size(mImage);

if mod(y,blockWidth) || mod(x,blockWidth);
    
    fprintf('A %ix%i image cannot be split exactly into square blocks of width, %i,\nResizing.........\n',y,x,blockWidth);
    nBlocks =floor(size(mImage)/blockWidth);
    y = nBlocks(1)*blockWidth;
    x = nBlocks(2)*blockWidth;
    if y == 0 || x == 0
        error('Image is too small to be split into square blocks!!!!');
    end
    mImage=mImage(1:y,1:x);
    fprintf('New image is  %ix%i pixels.\n',y,x);
    
end
%% Split the image into a cell array of square blocks
cImage = mat2cell(mImage, blockWidth*ones(1, (y/blockWidth)), ...
    blockWidth*ones(1, (x/blockWidth)));
[nYBlocks, nXBlocks]  = size(cImage);