function [mDictionary, vSize, mDictionaryKron] = Dictionary(type, sz1DSpace)
% Dictionary Constructs and returns the requested dictionary.
%
% Returns the Kronecker product of either:
% 1) The discrete cosine atoms spanning \(\mathbb{R}^{\mbox{sz1DSpace}}\) with themselves or,
% 2) The combined dictionary \([D_{a}, D_{b}]\) with itself where:
%   \(D_{a}\) - Normalized redundancy \(2\) discrete cosine dictionary.
%   \(D_{b}\) - Dirac basis.
%
% Usage:    [mDictionaryKron, mDictionary, vSize] = Dictionary( type, sz1DSpace );
%   
% Inputs:
%   type        DCT - Basis of discrete cosines for the space \(\mathbb{R}^{\mbox{sz1DSpace}}\).
%               RDCTDB - Dictionary \([D_{a}, D_{b}]\).
%   sz1DSpace   Dimension of atoms before applying the Kronecker product.
%
% Output:
%   mDictionaryKron Matrix containing the 2D dictionary.
%   mDictionary     Matrix containing the 1D dictionary.
%   vSize           Vector containing the sizes of each 1D dictionary. 

%% Main Routine
if strcmp(type, 'DCT')
    
    mDictionary = NormDict(DCos(sz1DSpace));
    vSize = sz1DSpace;
    
elseif strcmp(type(1:6), 'RDCTDB')
    
    mRDC = NormDict(DCos(sz1DSpace, sz1DSpace*2-1, 2));
    mEuclid = eye(sz1DSpace);
    mDictionary = [mRDC mEuclid];
    vSize = [sz1DSpace*2-1;sz1DSpace];
    
end
    
if nargout > 2
    mDictionaryKron = kron(mDictionary,mDictionary);    
end