function ind = KroneckerIndex(iD1,iD2,iV1,iV2,vSize)
% KroneckerIndex Given the indexes in <latex>D_a<latex> and <latex>D_b<latex> it returns the index in <latex>D\otimes D<latex> where <latex>D = [D_a, D_b]<latex>.
%
% Usage:    ind = KroneckerIndex( iD1, iD2, iV1, iV2, vSize );
%
% Inputs
%   iD1             Indices of the dictionary to which the selected atom belongs (w.r.t 
%                   columns) iD1 = 1 indicates that the selected atom belongs to dictionary
%                   \(D_{a}\).
%   iV1             Indices of the selected atom in the dictionary iD1.
%   iD2             Indices of the dictionary to which the selected atom belongs (w.r.t.
%                   rows).
%   iV2             Indices of the selected atom in the dictionary iD2.
%   vSize           Vector containing the sizes of each 1D dictionary. 
%
% Output
%   ind             Index in the larger dictionary D.

%% Main Routine
n = sum(vSize);
sz1 = cumsum(vSize);
sz2 = [0; sz1(1:end-1)];
ind = sz2(iD1)*n + (iV1-1)*n + sz2(iD2) + iV2;