function [mImage, sInfo, map] = LoadImage(imagePath)
% LoadImage Takes the path to an image file as input and returns a matrix representation of
% the image.


%
% Also returns a structure containing the image meta information and a map
% if the it is an indexed image.
%
% Usage:            [mImage, sInfo, map] = LoadImage(imagePath)
% Inputs:
%   imagePath       Relative or apsolute path to where the image file is
%                   stored, inclusing file name.
%
% Outputs:
%   mImage          matrix representation of the image
%   sInfo           structure containing the meta information included in
%                     the image file.
%   map             map of index's to intensities if indexed image
% 

%% Check if file exists
if exist(imagePath,'file') ~= 2
       
    error('%s not found in current directory or on MATLAB search path!',imagePath);
    
end
%% Assume its an image if imfinfo doesn't throw an exception
try
    
    sInfo = imfinfo(imagePath);
    
catch ME
    
    error('%s does not appear to be an image',imagePath);
    
end
%% Read the image into a matrix    
try
    
    [mImage, map] = imread(imagePath);
    
catch ME
    
    error('%s cannot be read!',imagePath);
    
end