function mSubImage = GetSubImage(mImage, iStartBlock, iEndBlock, blockWidth, rowScan)


% iStart and iEnd can be 1S or 2D index's assumes row scanning so nYBlocks
% is determined by the index's supplied.
% If nXBlocks is supplied then reduces the width of the sub image otherwise
% width will be the same as original
% returned image has the same
% Index's are in terms of the blockWidth
% Two modes of operation either gets a small square or scans rows from
% iStart to iEnd
%% Input Validation
if nargin < 3
    error('Not enough input arguments!');
end
if nargin < 4
    blockWidth = 1;
end
if nargin < 5
    rowScan = 1;
end
[nYPixelsOriginal, nXPixelsOriginal] = size(mImage);

% Check blockWidth is  valid
if mod(nYPixelsOriginal,blockWidth) == 0
    nYBlocks = nYPixelsOriginal/blockWidth;
else 
    error('Image cannot be split into blocks of width %i!',blockWidth);
end
if mod(nXPixelsOriginal,blockWidth) == 0
    nXBlocks = nXPixelsOriginal/blockWidth;
else 
    error('Image cannot be split into blocks of width %i!',blockWidth);
end
%% Get row and column index's and verify they are within the image
if numel(iStartBlock) == 1
    [iStartBlock(1) iStartBlock(2)] = ind2sub([nYBlocks, nXBlocks], iStartBlock);  
end
if numel(iEndBlock) == 1
    [iEndBlock(1) iEndBlock(2)] = ind2sub([nYBlocks, nXBlocks], iEndBlock);  
end

iStartPixel = (iStartBlock-1)*blockWidth+1;
iYStartPixel = iStartPixel(1);
iXStartPixel = iStartPixel(2);
if iYStartPixel < 0 || iXStartPixel < 0 || iYStartPixel > nYPixelsOriginal || iXStartPixel > nXPixelsOriginal
    error('The start index of the sub image is not within the image!');
end

iEndPixel = iEndBlock*blockWidth;
iYEndPixel = iEndPixel(1);
iXEndPixel = iEndPixel(2);
if iYEndPixel < 0 || iXEndPixel < 0 || iYEndPixel > nYPixelsOriginal || iXEndPixel > nXPixelsOriginal
    error('The end index of the sub image is not within the image!');
end
%% Get the new dimensions
nYPixels = iYEndPixel - iYStartPixel+1;
if rowScan
    nXPixels = nXPixelsOriginal;
else
    nXPixels = iXEndPixel - iXStartPixel+1;
end
%% Copy pixels to sub image container
mSubImage = zeros(nYPixels,nXPixels);

% Do first row
if rowScan
    if nYPixels == 1
        mSubImage(1:blockWidth,iXStartPixel:iXEndPixel) = mImage(iYStartPixel:iYStartPixel+blockWidth-1,iXStartPixel:iXEndPixel);
    else
        mSubImage(1:blockWidth,iXStartPixel:nXPixels) = mImage(iYStartPixel:iYStartPixel+blockWidth-1,iXStartPixel:nXPixels);
        mSubImage(blockWidth+1:nYPixels-blockWidth,:) = mImage(iYStartPixel+blockWidth:iYEndPixel-blockWidth,:);
        mSubImage(end-blockWidth+1:end,1:iXEndPixel) = mImage(iYEndPixel-blockWidth+1:iYEndPixel,1:iXEndPixel);
    end
else
    mSubImage = mImage(iYStartPixel:iYEndPixel,iXStartPixel:iXEndPixel);
end

