function vCnQuant = RRescaleCoefficients(vHiddenCn,vHiddenIndex,cQuant,displayAnalysis)
% RRescaleCoefficients Used by ExpandImage to rescale coefficients.

if nargin < 4
    displayAnalysis = 0;
end
%%  Make additional argument of old cn to measure distortion
method = cQuant.method;
 
if ~(strcmp(method,'minQuant') || strcmp(method,'maxQuant') || strcmp(method,'rescale') || strcmp(method,'none'))
    
    error('Quantization method unknown!');
    
end
if strcmp(method,'none')
    fprintf('No Quantization Required\n!');
else
    vQuantIndex =  cQuant.vQuantIndex;
    nQuant = numel(vQuantIndex);
    vCnQuant = vHiddenCn;
    vDistortion = zeros(nQuant,1);
    vNToQuant = zeros(nQuant,1);
    %% Quantize coefficient in vHiddenCn with index vQuantIndex in vHiddenIndex
    vMinCnRecovered = zeros(nQuant,1);
    vMaxCnRecovered = zeros(nQuant,1);
    vMeanCnRecovered = zeros(nQuant,1);
    if strcmp(method,'minQuant') || strcmp(method,'maxQuant')
        vQuantFactors = cQuant.rQuantFactors;
        for i =1:nQuant

            iCnToQuantize = (vHiddenIndex == vQuantIndex(i));
            vCnOld = vCnQuant(iCnToQuantize);
            vNToQuant(i) = numel(vCnOld);
            vCnNew = vCnOld/vQuantFactors(i);        
            vCnQuant(iCnToQuantize) = vCnNew;
            
            vCnNewSorted = sort(abs(vCnNew));            
            vMinCnRecovered(i) = vCnNewSorted(min(find(vCnNewSorted > 0)));
            clear vCnNewSorted;
            vMaxCnRecovered(i) = max(abs(vCnNew));
            vMeanCnRecovered(i) = mean(abs(vCnNew));

        end
    elseif strcmp(method,'rescale')

        mQuantFactors = cQuant.rQuantFactors;

        for i =1:nQuant

            iCnToQuantize = (vHiddenIndex == vQuantIndex(i));
            vCnOld = vCnQuant(iCnToQuantize);
            vNToQuant(i) = numel(vCnOld);
            vCnNew = Rescale(vCnOld,mQuantFactors(i,1),mQuantFactors(i,2));
            vCnQuant(iCnToQuantize) = vCnNew;

            vCnNewSorted = sort(abs(vCnNew));            
            vMinCnRecovered(i) = vCnNewSorted(min(find(vCnNewSorted > 0)));
            clear vCnNewSorted;
            vMaxCnRecovered(i) = max(abs(vCnNew));
            vMeanCnRecovered(i) = mean(abs(vCnNew));

        end

    end

    if displayAnalysis
        %% Output Quantization Results
        PrintWithStars('Quantization Results');
        vCnQuantSorted = sort(abs(vCnQuant)); % Ignore zero coefficients
        cnMinQuant = vCnQuantSorted(min(find(vCnQuantSorted > 0)));
        clear vCnQuantSorted;
        cnMaxQuant = max(abs(vCnQuant));
        cnMeanQuant = mean(abs(vCnQuant));
        % Print this out by index
        fprintf('Range of recovered coefficints\n');
        fprintf('Min: %.2f, Max %.2f, Mean: %.2f\n',cnMinQuant,cnMaxQuant,cnMeanQuant);

        fprintf('  Hidden Coefficinets By Index\n');
        for i = 1:nQuant
            fprintf('    Recovered :- Min: %.2f, Max %.2f, Mean: %.2f\n',vMinCnRecovered(i),vMaxCnRecovered(i),vMeanCnRecovered(i));
        end

    end
end
