function [ cRecoveredImage, cNoise, vCn] = ...
    SeparateCellImage(cFoldedImage, cRecoveredImage, mDictionary, blockWidth, iXBlockEnd, cIndex, sKey)
% SeparateCellImage Used by ExpandImage to 1) separate the host and embedded image using 
% SplitOrthComp and 2) retrieve the embedded coefficients using GenerateEncryptionOperator.

% Set up random matrices
%   Performed here to decrease processing time.
%   The public matrix should change for each block.
    % Private key - fixed matrix initialized to the largest possible size.
        RngW(sKey.privateKey);
        nRows = size(mDictionary,1);
        mRandPrivate =  rand(nRows-1,nRows -1) - 0.5;      
    % - - > >
    
    % Public key - initialize seed here
        RngW(sKey.publicKey);
    % - - > >
% - - > >

    % Get size of folded image
    [nYBlocks, nXBlocks] = size(cFoldedImage);
    nPixelsBlock = blockWidth^2;

    % Allocate memory for variables
    vCn = zeros(nPixelsBlock*nYBlocks*nXBlocks,1);
    %cRecoveredImage = cell(size(cIndex));
    cNoise = cell(nYBlocks,nXBlocks);
    
    %% Process first nYBlocks - 1 Rows
    iVCn = 1;
    for i = 1:nYBlocks - 1

        for j = 1:nXBlocks        

            % Remove zero indexes
                
            
            % Get the index's of the space to project onto (image space)
            iMDictionary = cIndex{i,j}.iMDictionary2D;

            % Remove the noise
            [ cRecoveredImage{i,j}, vNoise, mPOrthSpace] = SplitOrthComp(...
                cFoldedImage{i,j}, mDictionary(:,iMDictionary));        
            cNoise{i,j} = zeros(blockWidth);
            cNoise{i,j}(:) = vNoise;

            % Calculate orthogonal space used to recover hidden coefficients
            nCnHidden = nPixelsBlock - size(iMDictionary,2);
%             mOrthSpace = GenerateEncryptionOperator(mDictionary(:,iMDictionary),...
%                 nCnHidden,sKey, mPOrthSpace);
%             mOrthSpace = GenEncryptOp(mDictionary(:,iMDictionary),nCnHidden...
%                 ,mRandPublic,sKey.rotate,mPOrthSpace);   

            mOrthSpace = GenerateEncryptionOperator(mDictionary(:,iMDictionary),nCnHidden,...
                mRandPrivate,sKey.rotate,mPOrthSpace);

            % Recover image coefficients from the noise
            vCn(iVCn:iVCn+nCnHidden-1) = mOrthSpace'*vNoise;
            iVCn = iVCn + nCnHidden;

        end

    end
    %% Deal with the last row seperatly as their maybe empty blocks
    for j = 1:iXBlockEnd

        % Get the index's of the space to project onto (image space)
        iMDictionary = cIndex{nYBlocks,j}.iMDictionary2D;

        % Remove the noise
        [ cRecoveredImage{nYBlocks,j}, vNoise, mPOrthSpace] = ...
            SplitOrthComp(cFoldedImage{nYBlocks,j},...
            mDictionary(:,iMDictionary)); 
        cNoise{nYBlocks,j} = zeros(blockWidth);
        cNoise{nYBlocks,j}(:) = vNoise;

        % Calculate orthogonal space used to recover hidden coefficients
        nCnHidden = nPixelsBlock - size(iMDictionary,2);
%         mOrthSpace = GenerateEncryptionOperator(mDictionary(:,iMDictionary),...
%             nCnHidden,sKey, mPOrthSpace);
%         mOrthSpace = GenEncryptOp(mDictionary(:,iMDictionary),nCnHidden...
%                 ,mRandPublic,sKey.rotate,mPOrthSpace);
        mOrthSpace = GenerateEncryptionOperator(mDictionary(:,iMDictionary),nCnHidden,...
                mRandPrivate,sKey.rotate,mPOrthSpace);

        % Recover image coefficients from the noise
        vCn(iVCn:iVCn+nCnHidden-1) = mOrthSpace'*vNoise;
        iVCn = iVCn + nCnHidden;

    end
    %% Reduce the size of vCn so it only contains coefficients and padding
    vCn = vCn(1:iVCn-1);
    %% Return the noise as a matrix
    for i = iXBlockEnd+1:nXBlocks
        cNoise{nYBlocks,i} = zeros(blockWidth);
    end

end

%% Begin sub functions
function [ mOrth, vOrthComp, mPOrthSpace] = SplitOrthComp(mBlock, mImageSpace)

    %% Remove the noise with orthogonal projection
    [mOrthAtoms mRTemp] = qr(mImageSpace,0);
    mPOrthSpace = mOrthAtoms*mOrthAtoms';
    vBlock = mBlock(:);
    vOrth = (mPOrthSpace*vBlock);
    mOrth = zeros(size(mBlock));
    mOrth(:) = vOrth;
    %% Retrieve the noise
    vOrthComp = vBlock - vOrth;
    
end


