function mEmbeddedImage = GenerateEmbeddedImage(vCn, mDictionary, cIndex, iXBlockEnd, sKey)
% GenerateEmbeddedImage Used by FoldImage to embed coefficients in a generated image (c.f. 
% eq. 8 in [1]).

% % Set up random streams
%     % Publlic key - initialize seed here
%         streamPublic = RandStream('mt19937ar','Seed',sKey.publicKey);
%     % - - > >
%     % Private key - initialize seed here
%         streamPrivate = RandStream('mt19937ar','Seed',sKey.privateKey);
%     % - - > >
% % - - > > #

% Set up random matrices
%   Performed here to decrease processing time.
%   The public matrix should change for each block.
    % Private key - fixed matrix initialized to the largest possible size.
        RngW(sKey.privateKey);
        nRows = size(mDictionary,1);
        mRandPrivate =  rand(nRows-1,nRows -1) - 0.5;      
    % - - > >
    
    % Public key - initialize seed here
        RngW(sKey.publicKey);
    % - - > >
% - - > >
    
% Get size of hidden image
[nYBlocks,nXBlocks] = size(cIndex);

% Allocate memory for variables
cEmbeddedImage = cell(nYBlocks,nXBlocks);
blockWidth = sqrt(size(mDictionary,1));
nPixelsBlock = blockWidth^2;
%% Process first nYBlocks Rows
iVCn = 1;
for i = 1:nYBlocks - 1
    
    for j = 1:nXBlocks        
        
        % Calculate the orthogonal space used to hide the coefficients
        iMDictionary = cIndex{i,j}.iMDictionary2D;
        nCn = size(iMDictionary,2);
        nCnToHide = nPixelsBlock - nCn;
        mImageSpace = mDictionary(:,iMDictionary);
%         mOrthSpace = GenerateEncryptionOperator(mImageSpace,nCnToHide,sKey);
        mOrthSpace= GenerateEncryptionOperator(mImageSpace,nCnToHide,mRandPrivate,...
            sKey.rotate);
       
%         % Change the public random matrix
%             mRandPublic = mRandPublic + rndAdd;
%             rndAdd = rndAdd + rndAdd;
%         % - - > >
            
        % Get the coefficients to hide
        vCnToHide = vCn(iVCn:iVCn+nCnToHide-1);
        iVCn = iVCn + nCnToHide;
        
        % Create a new block in the orthogonal space
        vHiddenBlock = mOrthSpace*vCnToHide;
        cEmbeddedImage{i,j} = zeros(blockWidth);
        cEmbeddedImage{i,j}(:) = vHiddenBlock;
        
    end
    
end
%% Deal with the last row seperatly as their maybe empty blocks
for j = 1:iXBlockEnd
    
        % Calculate the orthogonal space used to hide the coefficients
        iMDictionary = cIndex{nYBlocks,j}.iMDictionary2D;
        nCn = size(iMDictionary,2);
        nCnToHide = nPixelsBlock - nCn;
        mImageSpace = mDictionary(:,iMDictionary);
%         mOrthSpace = GenerateEncryptionOperator(mImageSpace,nCnToHide,sKey);
        mOrthSpace = GenerateEncryptionOperator(mImageSpace,nCnToHide,mRandPrivate,...
            sKey.rotate);         
        
        % Get the coefficients to hide
        vCnToHide = vCn(iVCn:iVCn+nCnToHide-1);
        iVCn = iVCn + nCnToHide;
        
        % Create a new block in the orthogonal space
        vHiddenBlock = mOrthSpace*vCnToHide;
        cEmbeddedImage{nYBlocks,j} = zeros(blockWidth);
        cEmbeddedImage{nYBlocks,j}(:) = vHiddenBlock;
    
end
%% Pad any remaining blocks with zeros
for j = iXBlockEnd+1:nXBlocks
    
    cEmbeddedImage{nYBlocks,j} = zeros(blockWidth);
    
end
%% Convert cell image back to matrix
mEmbeddedImage = cell2mat(cEmbeddedImage);
