function vCnQuant = RescaleCoefficients(vHiddenCn,vHiddenIndex,cQuant,displayAnalysis)
% RescaleCoefficients Used by FoldImage to rescale coefficients.

if nargin < 4
    displayAnalysis = 0;
end
%%  Make additional argument of old cn to measure distortion
method = cQuant.method;
 
if ~(strcmp(method,'minQuant') || strcmp(method,'maxQuant') || strcmp(method,'rescale') || strcmp(method,'none'))
    
    error('Quantization method unknown!');
    
end
if strcmp(method,'none')
    fprintf('No Quantization Required\n!');
else
    vQuantIndex =  cQuant.vQuantIndex;
    nQuant = numel(vQuantIndex);
    vCnQuant = vHiddenCn;
    vDistortion = zeros(nQuant,1);
    vNToQuant = zeros(nQuant,1);
    %% Quantize coefficient in vHiddenCn with index vQuantIndex in vHiddenIndex
    if strcmp(method,'minQuant') || strcmp(method,'maxQuant')
        
        vQuantFactors = cQuant.quantFactors;

        for i =1:nQuant

            iCnToQuantize = (vHiddenIndex == vQuantIndex(i));
            vCnOld = vCnQuant(iCnToQuantize);
            vNToQuant(i) = numel(vCnOld);
            vCnNew = vCnOld/vQuantFactors(i);        
            vCnQuant(iCnToQuantize) = vCnNew;
            vDistortion(i) = sum(abs(vCnOld - round(vCnNew)*vQuantFactors(i)));

        end
    elseif strcmp(method,'rescale')
        mQuantFactors = cQuant.quantFactors;
        mRQuantFactors = cQuant.rQuantFactors;

        for i =1:nQuant

            iCnToQuantize = (vHiddenIndex == vQuantIndex(i));
            vCnOld = vCnQuant(iCnToQuantize);
            vNToQuant(i) = numel(vCnOld);
            vCnNew = Rescale(vCnOld,mQuantFactors(i,1),mQuantFactors(i,2));
            vCnQuant(iCnToQuantize) = vCnNew;
            vDistortion(i) = sum(abs(vCnOld - Rescale(round(vCnNew),mRQuantFactors(i,1),mRQuantFactors(i,2))));

        end

    end

    if displayAnalysis
        %% Output Quantization Results
        PrintWithStars('Quantization Results');
        vCnQuantSorted = sort(abs(vCnQuant)); % Ignore zero coefficients
        cnMinQuant = vCnQuantSorted(min(find(vCnQuantSorted > 0)));
        clear vCnQuantSorted;
        cnMaxQuant = max(abs(vCnQuant));
        cnMeanQuant = mean(abs(vCnQuant));
        % Print this out by index
        fprintf('New Range of hidden coefficints\n');
        fprintf('Min: %.2f, Max %.2f, Mean: %.2f\n',cnMinQuant,cnMaxQuant,cnMeanQuant);

        % Distortion by cn value
        fprintf('  Distortion By Index of quantization\n');
        for i = 1:nQuant
            fprintf('    Index: %3.0i, #Coefficients: %4.0i, Distortion: %.4f\n',cQuant.vQuantIndex(i),vNToQuant(i),vDistortion(i)/vNToQuant(i));
        end
        fprintf('  Total Distortion: %.4f\n',sum(vDistortion)/sum(vNToQuant));
        end

    end
end
